@@ -99,6 +99,7 @@ struct _GDBusMessage
 #ifdef G_OS_UNIX
   GUnixFDList *fd_list;
 #endif
+  GDBusMessageByteOrder byte_order;
 };
 
 G_DEFINE_TYPE (GDBusMessage, g_dbus_message, G_TYPE_OBJECT);
@@ -134,6 +135,18 @@ g_dbus_message_class_init (GDBusMessageClass *klass)
 static void
 g_dbus_message_init (GDBusMessage *message)
 {
+  /* Any D-Bus implementation is supposed to handle both Big and
+   * Little Endian encodings and the Endianness is part of the D-Bus
+   * message - we prefer to use Big Endian (since it's Network Byte
+   * Order and just easier to read for humans) but if the machine is
+   * Little Endian we use that for performance reasons.
+   */
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+  message->byte_order = G_DBUS_MESSAGE_BYTE_ORDER_LITTLE_ENDIAN;
+#else
+  /* this could also be G_PDP_ENDIAN */
+  message->byte_order = G_DBUS_MESSAGE_BYTE_ORDER_BIG_ENDIAN;
+#endif
   message->headers = g_hash_table_new_full (g_direct_hash,
                                             g_direct_equal,
                                             NULL,
@@ -197,7 +210,7 @@ g_dbus_message_new_method_call (const gchar *name,
 /**
  * g_dbus_message_new_signal:
  * @path: A valid object path.
- * @interface_: A valid D-Bus interface name or %NULL.
+ * @interface_: A valid D-Bus interface name.
  * @signal: A valid signal name.
  *
  * Creates a new #GDBusMessage for a signal emission.
@@ -215,7 +228,7 @@ g_dbus_message_new_signal (const gchar  *path,
 
   g_return_val_if_fail (g_variant_is_object_path (path), NULL);
   g_return_val_if_fail (g_dbus_is_member_name (signal), NULL);
-  g_return_val_if_fail (interface_ == NULL || g_dbus_is_interface_name (interface_), NULL);
+  g_return_val_if_fail (g_dbus_is_interface_name (interface_), NULL);
 
   message = g_dbus_message_new ();
   message->type = G_DBUS_MESSAGE_TYPE_SIGNAL;
@@ -223,9 +236,7 @@ g_dbus_message_new_signal (const gchar  *path,
 
   g_dbus_message_set_path (message, path);
   g_dbus_message_set_member (message, signal);
-
-  if (interface_ != NULL)
-    g_dbus_message_set_interface (message, interface_);
+  g_dbus_message_set_interface (message, interface_);
 
   return message;
 }
@@ -255,6 +266,8 @@ g_dbus_message_new_method_reply (GDBusMessage *method_call_message)
   message = g_dbus_message_new ();
   message->type = G_DBUS_MESSAGE_TYPE_METHOD_RETURN;
   message->flags = G_DBUS_MESSAGE_FLAGS_NO_REPLY_EXPECTED;
+  /* reply with same endianness */
+  message->byte_order = method_call_message->byte_order;
 
   g_dbus_message_set_reply_serial (message, g_dbus_message_get_serial (method_call_message));
   sender = g_dbus_message_get_sender (method_call_message);
@@ -327,6 +340,8 @@ g_dbus_message_new_method_error_literal (GDBusMessage  *method_call_message,
   message = g_dbus_message_new ();
   message->type = G_DBUS_MESSAGE_TYPE_ERROR;
   message->flags = G_DBUS_MESSAGE_FLAGS_NO_REPLY_EXPECTED;
+  /* reply with same endianness */
+  message->byte_order = method_call_message->byte_order;
 
   g_dbus_message_set_reply_serial (message, g_dbus_message_get_serial (method_call_message));
   g_dbus_message_set_error_name (message, error_name);
@@ -371,6 +386,38 @@ g_dbus_message_new_method_error_valist (GDBusMessage             *method_call_me
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+/**
+ * g_dbus_message_get_byte_order:
+ * @message: A #GDBusMessage.
+ *
+ * Gets the byte order of @message.
+ *
+ * Returns: The byte order.
+ */
+GDBusMessageByteOrder
+g_dbus_message_get_byte_order (GDBusMessage *message)
+{
+  g_return_val_if_fail (G_IS_DBUS_MESSAGE (message), (GDBusMessageByteOrder) 0);
+  return message->byte_order;
+}
+
+/**
+ * g_dbus_message_set_byte_order:
+ * @message: A #GDBusMessage.
+ * @byte_order: The byte order.
+ *
+ * Sets the byte order of @message.
+ */
+void
+g_dbus_message_set_byte_order (GDBusMessage          *message,
+                               GDBusMessageByteOrder  byte_order)
+{
+  g_return_if_fail (G_IS_DBUS_MESSAGE (message));
+  message->byte_order = byte_order;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 /* TODO: need GI annotations to specify that any guchar value goes for the type */
 
 /**
@@ -696,6 +743,105 @@ g_dbus_message_set_unix_fd_list (GDBusMessage  *message,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static gboolean
+validate_headers (GDBusMessage  *message,
+                  GError       **error)
+{
+  gboolean ret;
+
+  g_return_val_if_fail (G_IS_DBUS_MESSAGE (message), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  ret = FALSE;
+
+  switch (message->type)
+    {
+    case G_DBUS_MESSAGE_TYPE_INVALID:
+      g_set_error_literal (error,
+                           G_IO_ERROR,
+                           G_IO_ERROR_INVALID_ARGUMENT,
+                           _("type is INVALID"));
+      goto out;
+      break;
+
+    case G_DBUS_MESSAGE_TYPE_METHOD_CALL:
+      if (g_dbus_message_get_header (message, G_DBUS_MESSAGE_HEADER_FIELD_PATH) == NULL ||
+          g_dbus_message_get_header (message, G_DBUS_MESSAGE_HEADER_FIELD_MEMBER) == NULL)
+        {
+          g_set_error_literal (error,
+                               G_IO_ERROR,
+                               G_IO_ERROR_INVALID_ARGUMENT,
+                               _("METHOD_CALL message: PATH or MEMBER header field is missing"));
+          goto out;
+        }
+      break;
+
+    case G_DBUS_MESSAGE_TYPE_METHOD_RETURN:
+      if (g_dbus_message_get_header (message, G_DBUS_MESSAGE_HEADER_FIELD_REPLY_SERIAL) == NULL)
+        {
+          g_set_error_literal (error,
+                               G_IO_ERROR,
+                               G_IO_ERROR_INVALID_ARGUMENT,
+                               _("METHOD_RETURN message: REPLY_SERIAL header field is missing"));
+          goto out;
+        }
+      break;
+
+    case G_DBUS_MESSAGE_TYPE_ERROR:
+      if (g_dbus_message_get_header (message, G_DBUS_MESSAGE_HEADER_FIELD_ERROR_NAME) == NULL ||
+          g_dbus_message_get_header (message, G_DBUS_MESSAGE_HEADER_FIELD_REPLY_SERIAL) == NULL)
+        {
+          g_set_error_literal (error,
+                               G_IO_ERROR,
+                               G_IO_ERROR_INVALID_ARGUMENT,
+                               _("ERROR message: REPLY_SERIAL or ERROR_NAME header field is missing"));
+          goto out;
+        }
+      break;
+
+    case G_DBUS_MESSAGE_TYPE_SIGNAL:
+      if (g_dbus_message_get_header (message, G_DBUS_MESSAGE_HEADER_FIELD_PATH) == NULL ||
+          g_dbus_message_get_header (message, G_DBUS_MESSAGE_HEADER_FIELD_INTERFACE) == NULL ||
+          g_dbus_message_get_header (message, G_DBUS_MESSAGE_HEADER_FIELD_MEMBER) == NULL)
+        {
+          g_set_error_literal (error,
+                               G_IO_ERROR,
+                               G_IO_ERROR_INVALID_ARGUMENT,
+                               _("SIGNAL message: PATH, INTERFACE or MEMBER header field is missing"));
+          goto out;
+        }
+      if (g_strcmp0 (g_dbus_message_get_path (message), "/org/freedesktop/DBus/Local") == 0)
+        {
+          g_set_error_literal (error,
+                               G_IO_ERROR,
+                               G_IO_ERROR_INVALID_ARGUMENT,
+                               _("SIGNAL message: The PATH header field is using the reserved value /org/freedesktop/DBus/Local"));
+          goto out;
+        }
+      if (g_strcmp0 (g_dbus_message_get_interface (message), "org.freedesktop.DBus.Local") == 0)
+        {
+          g_set_error_literal (error,
+                               G_IO_ERROR,
+                               G_IO_ERROR_INVALID_ARGUMENT,
+                               _("SIGNAL message: The INTERFACE header field is using the reserved value org.freedesktop.DBus.Local"));
+          goto out;
+        }
+      break;
+
+    default:
+      /* hitherto unknown type - nothing to check */
+      break;
+    }
+
+  ret = TRUE;
+
+ out:
+  g_assert (ret || (error == NULL || *error != NULL));
+  return ret;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static gboolean
 ensure_input_padding (GMemoryInputStream   *mis,
                       gsize                 padding_size,
@@ -728,6 +874,7 @@ read_string (GMemoryInputStream    *mis,
   gsize remaining;
   guchar nul;
   GError *local_error;
+  const gchar *end_valid;
 
   s = g_string_new (NULL);
 
@@ -767,13 +914,30 @@ read_string (GMemoryInputStream    *mis,
       g_propagate_error (error, local_error);
       goto fail;
     }
+  if (!g_utf8_validate (s->str, -1, &end_valid))
+    {
+      gint offset;
+      gchar *valid_str;
+      offset = (gint) (end_valid - s->str);
+      valid_str = g_strndup (s->str, offset);
+      g_set_error (error,
+                   G_IO_ERROR,
+                   G_IO_ERROR_INVALID_ARGUMENT,
+                   _("Expected valid UTF-8 string but found invalid bytes at byte offset %d (length of string is %d). "
+                     "The valid UTF-8 string up until that that point was `%s'"),
+                   offset,
+                   (gint) s->len,
+                   valid_str);
+      g_free (valid_str);
+      goto fail;
+    }
   if (nul != '\0')
     {
       g_set_error (error,
                    G_IO_ERROR,
                    G_IO_ERROR_INVALID_ARGUMENT,
-                   _("Expected NUL byte after the string `%s' but found `%c' (%d)"),
-                   s->str, nul, nul);
+                   _("Expected NUL byte after the string `%s' but found byte %d"),
+                   s->str, nul);
       goto fail;
     }
 
@@ -929,7 +1093,7 @@ parse_value_from_blob (GMemoryInputStream    *mis,
           v = g_data_input_stream_read_uint64 (dis, NULL, &local_error);
           if (local_error != NULL)
             goto fail;
-          /* TODO: hmm */
+          G_STATIC_ASSERT (sizeof (gdouble) == sizeof (guint64));
           encoded = (gdouble *) &v;
           ret = g_variant_new_double (*encoded);
         }
@@ -1006,6 +1170,19 @@ parse_value_from_blob (GMemoryInputStream    *mis,
           g_free (v);
         }
     }
+  else if (g_variant_type_equal (type, G_VARIANT_TYPE_HANDLE))
+    {
+      if (!ensure_input_padding (mis, 4, &local_error))
+        goto fail;
+      if (!just_align)
+        {
+          gint32 v;
+          v = g_data_input_stream_read_int32 (dis, NULL, &local_error);
+          if (local_error != NULL)
+            goto fail;
+          ret = g_variant_new_handle (v);
+        }
+    }
   else if (g_variant_type_is_array (type))
     {
       guint32 array_len;
@@ -1038,7 +1215,7 @@ parse_value_from_blob (GMemoryInputStream    *mis,
               g_set_error (&local_error,
                            G_IO_ERROR,
                            G_IO_ERROR_INVALID_ARGUMENT,
-                           _("Encountered array of length %u bytes. Maximum length is 2<<26 bytes."),
+                           _("Encountered array of length %u bytes. Maximum length is 2<<26 bytes (64 MiB)."),
                            array_len);
               goto fail;
             }
@@ -1354,7 +1531,9 @@ g_dbus_message_bytes_needed (guchar                *blob,
  * @capabilities: A #GDBusCapabilityFlags describing what protocol features are supported.
  * @error: Return location for error or %NULL.
  *
- * Creates a new #GDBusMessage from the data stored at @blob.
+ * Creates a new #GDBusMessage from the data stored at @blob. The byte
+ * order that the message was in can be retrieved using
+ * g_dbus_message_get_byte_order().
  *
  * Returns: A new #GDBusMessage or %NULL if @error is set. Free with
  * g_object_unref().
@@ -1398,16 +1577,18 @@ g_dbus_message_new_from_blob (guchar                *blob,
     {
     case 'l':
       byte_order = G_DATA_STREAM_BYTE_ORDER_LITTLE_ENDIAN;
+      message->byte_order = G_DBUS_MESSAGE_BYTE_ORDER_LITTLE_ENDIAN;
       break;
     case 'B':
       byte_order = G_DATA_STREAM_BYTE_ORDER_BIG_ENDIAN;
+      message->byte_order = G_DBUS_MESSAGE_BYTE_ORDER_BIG_ENDIAN;
       break;
     default:
       g_set_error (error,
                    G_IO_ERROR,
                    G_IO_ERROR_INVALID_ARGUMENT,
-                   _("Invalid endianness value. Expected 'l' or 'B' but found '%c' (%d)"),
-                   endianness, endianness);
+                   _("Invalid endianness value. Expected 0x6c ('l') or 0x42 ('B') but found value 0x%02x"),
+                   endianness);
       goto out;
     }
   g_data_input_stream_set_byte_order (dis, byte_order);
@@ -1527,6 +1708,11 @@ g_dbus_message_new_from_blob (guchar                *blob,
         }
     }
 
+  if (!validate_headers (message, error))
+    {
+      g_prefix_error (error, _("Cannot deserialize message: "));
+      goto out;
+    }
 
   ret = TRUE;
 
@@ -1659,7 +1845,7 @@ append_value_to_blob (GVariant             *value,
         {
           guint64 *encoded;
           gdouble v = g_variant_get_double (value);
-          /* TODO: hmm */
+          G_STATIC_ASSERT (sizeof (gdouble) == sizeof (guint64));
           encoded = (guint64 *) &v;
           g_data_output_stream_put_uint64 (dos, *encoded, NULL, NULL);
         }
@@ -1670,7 +1856,10 @@ append_value_to_blob (GVariant             *value,
       if (value != NULL)
         {
           gsize len;
-          const gchar *v = g_variant_get_string (value, &len);
+          const gchar *v;
+          const gchar *end;
+          v = g_variant_get_string (value, &len);
+          g_assert (g_utf8_validate (v, -1, &end) && (end == v + len));
           g_data_output_stream_put_uint32 (dos, len, NULL, NULL);
           g_data_output_stream_put_string (dos, v, NULL, NULL);
           g_data_output_stream_put_byte (dos, '\0', NULL, NULL);
@@ -1681,9 +1870,9 @@ append_value_to_blob (GVariant             *value,
       padding_added = ensure_output_padding (mos, dos, 4);
       if (value != NULL)
         {
-          /* TODO: validate object path */
           gsize len;
           const gchar *v = g_variant_get_string (value, &len);
+          g_assert (g_variant_is_object_path (v));
           g_data_output_stream_put_uint32 (dos, len, NULL, NULL);
           g_data_output_stream_put_string (dos, v, NULL, NULL);
           g_data_output_stream_put_byte (dos, '\0', NULL, NULL);
@@ -1693,14 +1882,23 @@ append_value_to_blob (GVariant             *value,
     {
       if (value != NULL)
         {
-          /* TODO: validate signature (including max len being 255) */
           gsize len;
           const gchar *v = g_variant_get_string (value, &len);
+          g_assert (g_variant_is_signature (v));
           g_data_output_stream_put_byte (dos, len, NULL, NULL);
           g_data_output_stream_put_string (dos, v, NULL, NULL);
           g_data_output_stream_put_byte (dos, '\0', NULL, NULL);
         }
     }
+  else if (g_variant_type_equal (type, G_VARIANT_TYPE_HANDLE))
+    {
+      padding_added = ensure_output_padding (mos, dos, 4);
+      if (value != NULL)
+        {
+          gint32 v = g_variant_get_handle (value);
+          g_data_output_stream_put_int32 (dos, v, NULL, NULL);
+        }
+    }
   else if (g_variant_type_is_array (type))
     {
       GVariant *item;
@@ -1816,7 +2014,6 @@ append_value_to_blob (GVariant             *value,
           const gchar *signature;
           child = g_variant_get_child_value (value, 0);
           signature = g_variant_get_type_string (child);
-          /* TODO: validate signature (including max len being 255) */
           g_data_output_stream_put_byte (dos, strlen (signature), NULL, NULL);
           g_data_output_stream_put_string (dos, signature, NULL, NULL);
           g_data_output_stream_put_byte (dos, '\0', NULL, NULL);
@@ -1903,7 +2100,8 @@ append_body_to_blob (GVariant             *value,
  * @capabilities: A #GDBusCapabilityFlags describing what protocol features are supported.
  * @error: Return location for error.
  *
- * Serializes @message to a blob.
+ * Serializes @message to a blob. The byte order returned by
+ * g_dbus_message_get_byte_order() will be used.
  *
  * Returns: A pointer to a valid binary D-Bus message of @out_size bytes
  * generated by @message or %NULL if @error is set. Free with g_free().
@@ -1945,12 +2143,19 @@ g_dbus_message_to_blob (GDBusMessage          *message,
   mos = G_MEMORY_OUTPUT_STREAM (g_memory_output_stream_new (NULL, 0, g_realloc, g_free));
   dos = g_data_output_stream_new (G_OUTPUT_STREAM (mos));
 
-  /* TODO: detect endianess... */
-  byte_order = G_DATA_STREAM_BYTE_ORDER_LITTLE_ENDIAN;
+  switch (message->byte_order)
+    {
+    case G_DBUS_MESSAGE_BYTE_ORDER_BIG_ENDIAN:
+      byte_order = G_DATA_STREAM_BYTE_ORDER_BIG_ENDIAN;
+      break;
+    case G_DBUS_MESSAGE_BYTE_ORDER_LITTLE_ENDIAN:
+      byte_order = G_DATA_STREAM_BYTE_ORDER_LITTLE_ENDIAN;
+      break;
+    }
   g_data_output_stream_set_byte_order (dos, byte_order);
 
   /* Core header */
-  g_data_output_stream_put_byte (dos, byte_order == G_DATA_STREAM_BYTE_ORDER_LITTLE_ENDIAN ? 'l' : 'B', NULL, NULL);
+  g_data_output_stream_put_byte (dos, (guchar) message->byte_order, NULL, NULL);
   g_data_output_stream_put_byte (dos, message->type, NULL, NULL);
   g_data_output_stream_put_byte (dos, message->flags, NULL, NULL);
   g_data_output_stream_put_byte (dos, 1, NULL, NULL); /* major protocol version */
@@ -1965,7 +2170,6 @@ g_dbus_message_to_blob (GDBusMessage          *message,
     num_fds_in_message = g_unix_fd_list_get_length (message->fd_list);
 #endif
   num_fds_according_to_header = g_dbus_message_get_num_unix_fds (message);
-  /* TODO: check we have all the right header fields and that they are the correct value etc etc */
   if (num_fds_in_message != num_fds_according_to_header)
     {
       g_set_error (error,
@@ -1977,6 +2181,12 @@ g_dbus_message_to_blob (GDBusMessage          *message,
       goto out;
     }
 
+  if (!validate_headers (message, error))
+    {
+      g_prefix_error (error, _("Cannot serialize message: "));
+      goto out;
+    }
+
   g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{yv}"));
   g_hash_table_iter_init (&hash_iter, message->headers);
   while (g_hash_table_iter_next (&hash_iter, &key, (gpointer) &header_value))
@@ -2038,7 +2248,7 @@ g_dbus_message_to_blob (GDBusMessage          *message,
     }
   else
     {
-      if (signature != NULL)
+      if (signature != NULL && strlen (signature_str) > 0)
         {
           g_set_error (error,
                        G_IO_ERROR,
