@@ -40,7 +40,6 @@
 #include "gcancellable.h"
 
 #include "glibintl.h"
-#include "gioalias.h"
 
 /**
  * SECTION:gdbusproxy
@@ -540,6 +539,13 @@ g_dbus_proxy_init (GDBusProxy *proxy)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static gint
+property_name_sort_func (const gchar **a,
+                         const gchar **b)
+{
+  return g_strcmp0 (*a, *b);
+}
+
 /**
  * g_dbus_proxy_get_cached_property_names:
  * @proxy: A #GDBusProxy.
@@ -570,7 +576,7 @@ g_dbus_proxy_get_cached_property_names (GDBusProxy  *proxy)
   g_hash_table_iter_init (&iter, proxy->priv->properties);
   while (g_hash_table_iter_next (&iter, (gpointer) &key, NULL))
     g_ptr_array_add (p, g_strdup (key));
-  g_ptr_array_sort (p, (GCompareFunc) g_strcmp0);
+  g_ptr_array_sort (p, (GCompareFunc) property_name_sort_func);
   g_ptr_array_add (p, NULL);
 
   names = (gchar **) g_ptr_array_free (p, FALSE);
@@ -738,7 +744,7 @@ on_signal_received (GDBusConnection *connection,
   if (!proxy->priv->initialized)
     goto out;
 
-  if (g_strcmp0 (sender_name, proxy->priv->name_owner) != 0)
+  if (proxy->priv->name_owner != NULL && g_strcmp0 (sender_name, proxy->priv->name_owner) != 0)
     goto out;
 
   g_signal_emit (proxy,
@@ -779,7 +785,7 @@ on_properties_changed (GDBusConnection *connection,
   if (!proxy->priv->initialized)
     goto out;
 
-  if (g_strcmp0 (sender_name, proxy->priv->name_owner) != 0)
+  if (proxy->priv->name_owner != NULL && g_strcmp0 (sender_name, proxy->priv->name_owner) != 0)
     goto out;
 
   if (!g_variant_is_of_type (parameters, G_VARIANT_TYPE ("(sa{sv}as)")))
@@ -833,7 +839,7 @@ process_get_all_reply (GDBusProxy *proxy,
   gchar *key;
   GVariant *value;
 
-  if (strcmp (g_variant_get_type_string (result), "(a{sv})") != 0)
+  if (!g_variant_is_of_type (result, G_VARIANT_TYPE ("(a{sv})")))
     {
       g_warning ("Value for GetAll reply with type `%s' does not match `(a{sv})'",
                  g_variant_get_type_string (result));
@@ -843,14 +849,28 @@ process_get_all_reply (GDBusProxy *proxy,
   g_variant_get (result, "(a{sv})", &iter);
   while (g_variant_iter_next (iter, "{sv}", &key, &value))
     {
-      /*g_print ("got %s -> %s\n", key, g_variant_print (value, FALSE));*/
-
       g_hash_table_insert (proxy->priv->properties,
                            key, /* adopts string */
                            value); /* adopts value */
     }
   g_variant_iter_free (iter);
 
+  /* Synthesize ::g-properties-changed changed */
+  if (g_hash_table_size (proxy->priv->properties) > 0)
+    {
+      GVariant *changed_properties;
+      const gchar *invalidated_properties[1] = {NULL};
+
+      g_variant_get (result,
+                     "(@a{sv})",
+                     &changed_properties);
+      g_signal_emit (proxy, signals[PROPERTIES_CHANGED_SIGNAL],
+                     0,
+                     changed_properties,
+                     invalidated_properties);
+      g_variant_unref (changed_properties);
+    }
+
  out:
   ;
 }
@@ -932,7 +952,6 @@ on_name_owner_changed (GDBusConnection *connection,
                        gpointer          user_data)
 {
   GDBusProxy *proxy = G_DBUS_PROXY (user_data);
-  const gchar *name;
   const gchar *old_owner;
   const gchar *new_owner;
 
@@ -945,19 +964,45 @@ on_name_owner_changed (GDBusConnection *connection,
 
   g_variant_get (parameters,
                  "(&s&s&s)",
-                 &name,
+                 NULL,
                  &old_owner,
                  &new_owner);
 
-  /* we only care about a specific name */
-  if (g_strcmp0 (name, proxy->priv->name) != 0)
-    goto out;
-
   if (strlen (new_owner) == 0)
     {
       g_free (proxy->priv->name_owner);
       proxy->priv->name_owner = NULL;
-      g_hash_table_remove_all (proxy->priv->properties);
+
+      /* Synthesize ::g-properties-changed changed */
+      if (!(proxy->priv->flags & G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES) &&
+          g_hash_table_size (proxy->priv->properties) > 0)
+        {
+          GVariantBuilder builder;
+          GVariant *changed_properties;
+          GPtrArray *invalidated_properties;
+          GHashTableIter iter;
+          const gchar *key;
+
+          /* Build changed_properties (always empty) and invalidated_properties ... */
+          g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
+          changed_properties = g_variant_builder_end (&builder);
+          invalidated_properties = g_ptr_array_new_with_free_func (g_free);
+          g_hash_table_iter_init (&iter, proxy->priv->properties);
+          while (g_hash_table_iter_next (&iter, (gpointer) &key, NULL))
+            g_ptr_array_add (invalidated_properties, g_strdup (key));
+          g_ptr_array_add (invalidated_properties, NULL);
+
+          /* ... throw out the properties ... */
+          g_hash_table_remove_all (proxy->priv->properties);
+
+          /* ... and finally emit the ::g-properties-changed signal */
+          g_signal_emit (proxy, signals[PROPERTIES_CHANGED_SIGNAL],
+                         0,
+                         changed_properties,
+                         (const gchar* const *) invalidated_properties->pdata);
+          g_variant_unref (changed_properties);
+          g_ptr_array_unref (invalidated_properties);
+        }
       g_object_notify (G_OBJECT (proxy), "g-name-owner");
     }
   else
@@ -1328,6 +1373,7 @@ async_initable_init_first (GAsyncInitable *initable)
                                             "PropertiesChanged",
                                             proxy->priv->object_path,
                                             proxy->priv->interface_name,
+                                            G_DBUS_SIGNAL_FLAGS_NONE,
                                             on_properties_changed,
                                             proxy,
                                             NULL);
@@ -1343,6 +1389,7 @@ async_initable_init_first (GAsyncInitable *initable)
                                             NULL,                        /* member */
                                             proxy->priv->object_path,
                                             NULL,                        /* arg0 */
+                                            G_DBUS_SIGNAL_FLAGS_NONE,
                                             on_signal_received,
                                             proxy,
                                             NULL);
@@ -1357,6 +1404,7 @@ async_initable_init_first (GAsyncInitable *initable)
                                             "NameOwnerChanged",      /* signal name */
                                             "/org/freedesktop/DBus", /* path */
                                             proxy->priv->name,       /* arg0 */
+                                            G_DBUS_SIGNAL_FLAGS_NONE,
                                             on_name_owner_changed,
                                             proxy,
                                             NULL);
@@ -1663,12 +1711,12 @@ g_dbus_proxy_new_finish (GAsyncResult  *res,
  * g_dbus_proxy_new_sync:
  * @connection: A #GDBusConnection.
  * @flags: Flags used when constructing the proxy.
- * @info: A #GDBusInterfaceInfo specifying the minimal interface that @proxy conforms to or %NULL.
- * @name: A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
+ * @info: (allow-none): A #GDBusInterfaceInfo specifying the minimal interface that @proxy conforms to or %NULL.
+ * @name: (allow-none): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
  * @object_path: An object path.
  * @interface_name: A D-Bus interface name.
- * @cancellable: A #GCancellable or %NULL.
- * @error: Return location for error or %NULL.
+ * @cancellable: (allow-none): A #GCancellable or %NULL.
+ * @error: (allow-none): Return location for error or %NULL.
  *
  * Creates a proxy for accessing @interface_name on the remote object
  * at @object_path owned by @name at @connection and synchronously
@@ -2109,6 +2157,7 @@ reply_cb (GDBusConnection *connection,
 
   /* no need to complete in idle since the method GDBusConnection already does */
   g_simple_async_result_complete (simple);
+  g_object_unref (simple);
 }
 
 static const GDBusMethodInfo *
@@ -2399,6 +2448,3 @@ g_dbus_proxy_call_sync (GDBusProxy      *proxy,
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
-
-#define __G_DBUS_PROXY_C__
-#include "gioaliasdef.c"
