@@ -29,22 +29,15 @@
 #include "gasyncresult.h"
 #include "ginetaddress.h"
 #include "ginetsocketaddress.h"
+#include "gnetworkingprivate.h"
 #include "gresolver.h"
 #include "gsimpleasyncresult.h"
 #include "gsocketaddressenumerator.h"
 #include "gioerror.h"
 #include "gsocketconnectable.h"
 
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
-#ifdef HAVE_WINSOCK2_H
-#include <winsock2.h>
-#endif
-
 #include <string.h>
 
-#include "gioalias.h"
 
 /**
  * SECTION:gnetworkaddress
@@ -166,8 +159,7 @@ g_network_address_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_HOSTNAME:
-      if (addr->priv->hostname)
-        g_free (addr->priv->hostname);
+      g_free (addr->priv->hostname);
       addr->priv->hostname = g_value_dup_string (value);
       break;
 
@@ -549,7 +541,6 @@ g_network_address_address_enumerator_next_async (GSocketAddressEnumerator  *enum
   GNetworkAddressAddressEnumerator *addr_enum =
     G_NETWORK_ADDRESS_ADDRESS_ENUMERATOR (enumerator);
   GSimpleAsyncResult *simple;
-  GSocketAddress *sockaddr;
 
   simple = g_simple_async_result_new (G_OBJECT (enumerator),
                                       callback, user_data,
@@ -567,10 +558,6 @@ g_network_address_address_enumerator_next_async (GSocketAddressEnumerator  *enum
     }
   else
     {
-      sockaddr = g_network_address_address_enumerator_next (enumerator, NULL, NULL);
-      if (sockaddr)
-        g_simple_async_result_set_op_res_gpointer (simple, sockaddr, g_object_unref);
-
       g_simple_async_result_complete_in_idle (simple);
       g_object_unref (simple);
     }
@@ -626,6 +613,3 @@ g_network_address_connectable_enumerate (GSocketConnectable *connectable)
 
   return (GSocketAddressEnumerator *)addr_enum;
 }
-
-#define __G_NETWORK_ADDRESS_C__
-#include "gioaliasdef.c"
