@@ -38,19 +38,18 @@
 #include "gasynchelper.h"
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:gunixoutputstream
- * @short_description: Streaming output operations for Unix file descriptors
+ * @short_description: Streaming output operations for UNIX file descriptors
  * @include: gio/gunixoutputstream.h
  * @see_also: #GOutputStream
  *
  * #GUnixOutputStream implements #GOutputStream for writing to a
- * unix file descriptor, including asynchronous operations. The file
+ * UNIX file descriptor, including asynchronous operations. The file
  * descriptor must be selectable, so it doesn't work with opened files.
  *
- * Note that <filename>&lt;gio/gunixoutputstream.h&gt;</filename> belongs 
+ * Note that <filename>&lt;gio/gunixoutputstream.h&gt;</filename> belongs
  * to the UNIX-specific GIO interfaces, thus you have to use the
  * <filename>gio-unix-2.0.pc</filename> pkg-config file when using it.
  */
@@ -488,6 +487,7 @@ g_unix_output_stream_write_async (GOutputStream       *stream,
   source = _g_fd_source_new (unix_stream->priv->fd,
 			     G_IO_OUT,
 			     cancellable);
+  g_source_set_name (source, "GUnixOutputStream");
   
   g_source_set_callback (source, (GSourceFunc)write_async_cb, data, g_free);
   g_source_attach (source, g_main_context_get_thread_default ());
@@ -599,6 +599,3 @@ g_unix_output_stream_close_finish (GOutputStream  *stream,
   /* Failures handled in generic close_finish code */
   return TRUE;
 }
-
-#define __G_UNIX_OUTPUT_STREAM_C__
-#include "gioaliasdef.c"
