@@ -125,6 +125,37 @@ test_methods (GDBusProxy *proxy)
   g_dbus_proxy_set_default_timeout (proxy, -1);
 }
 
+static gboolean
+strv_equal (gchar **strv, ...)
+{
+  gint count;
+  va_list list;
+  const gchar *str;
+  gboolean res;
+
+  res = TRUE;
+  count = 0;
+  va_start (list, strv);
+  while (1)
+    {
+      str = va_arg (list, const gchar *);
+      if (str == NULL)
+        break;
+      if (g_strcmp0 (str, strv[count]) != 0)
+        {
+          res = FALSE;
+          break;
+        }
+      count++;
+    }
+  va_end (list);
+
+  if (res)
+    res = g_strv_length (strv) == count;
+
+  return res;
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 /* Test that the property aspects of GDBusProxy works */
 /* ---------------------------------------------------------------------------------------------------- */
@@ -136,9 +167,50 @@ test_properties (GDBusProxy *proxy)
   GVariant *variant;
   GVariant *variant2;
   GVariant *result;
+  gchar **names;
 
   error = NULL;
 
+  if (g_dbus_proxy_get_flags (proxy) & G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES)
+    {
+       g_assert (g_dbus_proxy_get_cached_property_names (proxy) == NULL);
+       return;
+    }
+
+  /*
+   * Check that we can list all cached properties.
+   */
+  names = g_dbus_proxy_get_cached_property_names (proxy);
+
+  g_assert (strv_equal (names,
+                        "PropertyThatWillBeInvalidated",
+                        "ab",
+                        "ad",
+                        "ai",
+                        "an",
+                        "ao",
+                        "aq",
+                        "as",
+                        "at",
+                        "au",
+                        "ax",
+                        "ay",
+                        "b",
+                        "d",
+                        "foo",
+                        "i",
+                        "n",
+                        "o",
+                        "q",
+                        "s",
+                        "t",
+                        "u",
+                        "x",
+                        "y",
+                        NULL));
+
+  g_strfreev (names);
+
   /*
    * Check that we can read cached properties.
    *
@@ -425,6 +497,51 @@ test_expected_interface (GDBusProxy *proxy)
   g_dbus_proxy_set_cached_property (proxy, "y", g_variant_new_byte (42));
 }
 
+static void
+test_basic (GDBusProxy *proxy)
+{
+  GDBusConnection *connection;
+  GDBusConnection *conn;
+  GDBusProxyFlags flags;
+  GDBusInterfaceInfo *info;
+  const gchar *name;
+  const gchar *path;
+  const gchar *interface;
+  gint timeout;
+
+  connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
+
+  g_assert (g_dbus_proxy_get_connection (proxy) == connection);
+  g_assert (g_dbus_proxy_get_flags (proxy) == G_DBUS_PROXY_FLAGS_NONE);
+  g_assert (g_dbus_proxy_get_interface_info (proxy) == NULL);
+  g_assert_cmpstr (g_dbus_proxy_get_name (proxy), ==, "com.example.TestService");
+  g_assert_cmpstr (g_dbus_proxy_get_object_path (proxy), ==, "/com/example/TestObject");
+  g_assert_cmpstr (g_dbus_proxy_get_interface_name (proxy), ==, "com.example.Frob");
+  g_assert_cmpint (g_dbus_proxy_get_default_timeout (proxy), ==, -1);
+
+  g_object_get (proxy,
+                "g-connection", &conn,
+                "g-interface-info", &info,
+                "g-flags", &flags,
+                "g-name", &name,
+                "g-object-path", &path,
+                "g-interface-name", &interface,
+                "g-default-timeout", &timeout,
+                NULL);
+
+  g_assert (conn == connection);
+  g_assert (info == NULL);
+  g_assert_cmpint (flags, ==, G_DBUS_PROXY_FLAGS_NONE);
+  g_assert_cmpstr (name, ==, "com.example.TestService");
+  g_assert_cmpstr (path, ==, "/com/example/TestObject");
+  g_assert_cmpstr (interface, ==, "com.example.Frob");
+  g_assert_cmpint (timeout, ==, -1);
+
+  g_object_unref (conn);
+
+  g_object_unref (connection);
+}
+
 static void
 test_proxy (void)
 {
@@ -460,6 +577,7 @@ test_proxy (void)
 
   _g_assert_property_notify (proxy, "g-name-owner");
 
+  test_basic (proxy);
   test_methods (proxy);
   test_properties (proxy);
   test_signals (proxy);
@@ -471,6 +589,63 @@ test_proxy (void)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static void
+proxy_ready (GObject      *source,
+             GAsyncResult *result,
+             gpointer      user_data)
+{
+  GDBusProxy *proxy;
+  GError *error;
+
+  error = NULL;
+  proxy = g_dbus_proxy_new_for_bus_finish (result, &error);
+  g_assert_no_error (error);
+
+  test_basic (proxy);
+  test_methods (proxy);
+  test_properties (proxy);
+  test_signals (proxy);
+  test_expected_interface (proxy);
+
+  g_object_unref (proxy);
+}
+
+static void
+test_async (void)
+{
+  g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+                            G_DBUS_PROXY_FLAGS_NONE,
+                            NULL,                      /* GDBusInterfaceInfo */
+                            "com.example.TestService", /* name */
+                            "/com/example/TestObject", /* object path */
+                            "com.example.Frob",        /* interface */
+                            NULL, /* GCancellable */
+                            proxy_ready,
+                            NULL);
+}
+
+static void
+test_no_properties (void)
+{
+  GDBusProxy *proxy;
+  GError *error;
+
+  error = NULL;
+  proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                         G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
+                                         NULL,                      /* GDBusInterfaceInfo */
+                                         "com.example.TestService", /* name */
+                                         "/com/example/TestObject", /* object path */
+                                         "com.example.Frob",        /* interface */
+                                         NULL, /* GCancellable */
+                                         &error);
+  g_assert_no_error (error);
+
+  test_properties (proxy);
+
+  g_object_unref (proxy);
+}
+
 int
 main (int   argc,
       char *argv[])
@@ -495,6 +670,8 @@ main (int   argc,
   g_setenv ("DBUS_SESSION_BUS_ADDRESS", session_bus_get_temporary_address (), TRUE);
 
   g_test_add_func ("/gdbus/proxy", test_proxy);
+  g_test_add_func ("/gdbus/proxy/async", test_async);
+  g_test_add_func ("/gdbus/proxy/no-properties", test_no_properties);
 
   ret = g_test_run();
 
