@@ -34,7 +34,6 @@
 #include "config.h"
 
 #include "glib.h"
-#include "galias.h"
 
 
 /* --- functions --- */
@@ -633,6 +632,3 @@ g_hook_compare_ids (GHook *new_hook,
   
   return 0;
 }
-
-#define __G_HOOK_C__
-#include "galiasdef.c"
